require('reshape2')
require('ggplot2')
require('grid')
require('gridExtra')

##############################################################
# 1. Generate 16s stacked bar plot from Stirrups output file
#    Input Format:
# 	SAMPLENAME|TAXA|AT|COUNT|PERCENTAGE|CONFIDENCE
##############################################################

##############################################################
# Read input files
##############################################################

# cytokine data
cytokines = read.table("MOMS-PI_cytokine_profiles_POP1.txt", header=T, sep="\t")
cytokines[cytokines=="OOR <"] = NaN
cytokines[cytokines=="OOR >"] = NaN
cytokines[cytokines==""] = NaN
kit_ids = sapply(cytokines$SampleID, function(x) substr(as.character(x), 1, 12))
cytokines$SampleID = kit_ids
cytokine_kits = unique(kit_ids)

# lipid data
lipids = read.table("MOMS-PI_lipidomics_profiles_POP1.txt", header=T, sep="\t")
kit_ids = sapply(lipids$SampleID, function(x) substr(as.character(x), 1, 12))
lipids$SampleID = kit_ids
lipid_kits = unique(kit_ids)

# 16s data
input.file = 'MOMS-PI_16s_STIRRUPS_profiles_POP1.txt'
raw = read.table(input.file, header=F, sep='|', skip=1)
data = subset(raw, V3=="AT", drop = T)
data = droplevels(data)
kit_ids = sapply(data$V1, function(x) substr(as.character(x), 1, 12))
data$V1 = kit_ids
stirrups_kits = unique(kit_ids)

# intersect kit lists to keep only kits contained in all data types
final_kit_list = intersect(intersect(cytokine_kits,lipid_kits),stirrups_kits)

# filter each data set for final kit list
tmp = data[data$V1 %in% final_kit_list, ] #subset(data, V1 %in% final_kit_list, drop=T)
tmp = droplevels(tmp)
data = tmp

tmp = cytokines[cytokines$SampleID %in% final_kit_list, ] # subset(cytokines, SampleID %in% final_kit_list, drop=T)
tmp = droplevels(tmp)
cytokines = tmp

tmp = lipids[lipids$SampleID %in% final_kit_list, ] # subset(lipids, SampleID %in% final_kit_list, drop=T)
tmp = droplevels(tmp)
lipids = tmp

##############################################################
# Determine taxa order and sample order
##############################################################
maxVals = do.call(rbind,lapply(split(data,data$V1),function(chunk) chunk[which.max(chunk$V5),]))
#taxaorder = unique(maxVals[order(-maxVals$V5),2])
taxaorder = unique(maxVals[order(-as.numeric(as.character(maxVals$V5))),2])

maxVals$V2 = factor(maxVals$V2, levels=taxaorder)
sampleorder = maxVals[order(maxVals$V2, -as.numeric(as.character(maxVals$V5))),]$V1
data$V1 = factor(data$V1, levels=sampleorder)

taxa.avgs = aggregate(as.numeric(as.character(data$V5)), by=list(data$V2), FUN=mean)
top.taxa = taxa.avgs[order(-taxa.avgs$x),][1:10,]$Group.1
top.taxa = droplevels(top.taxa)

##############################################################
# Determine bar colors - unspecified taxa are black
##############################################################
data$color = NA
taxa.colors = c("hotpink", "green", "orange", "red", "blue", "yellow", "purple", "lightpink", "lightblue", "black", "maroon")
for (i in 1:nrow(data)) {
 if (is.element(data[i,2], top.taxa)) {    
  data[i,]$color = as.character(data[i,2])
 }
 else {
  data[i,]$color = "Other"
 }
}

##############################################################
# Format data and create 16s graph
##############################################################
sbars = ggplot(data=data, aes(x = V1, y = V5,fill=color)) +
               geom_bar(stat='identity') +
               ylab("16s Profiles") +
               scale_fill_manual(values=taxa.colors) +
               ggtitle("MOMS-PI POP1 Integrated Dataset") + 
#               theme(axis.text.x = element_text(angle = 90, hjust = 1), legend.title=element_blank(), legend.position="bottom", axis.title.x = element_blank())
               theme(axis.text.x = element_blank(), legend.position="top", legend.title=element_blank(), axis.title.x = element_blank()) 

##############################################################
# 2. Generate heatmap for cytokine data
#    Input Format:
#       SampleID Cytokine Cytokine.Concentration Protein.Concentration Normalized.Protein.Concentration
##############################################################

# reshape data into a table
cytokinesMatrix=dcast(cytokines, SampleID ~ Cytokine, value.var='Normalized.Protein.Concentration.pg.cytokine.mg.protein.')

# remove rows with only NA values
#ind = apply(cytokinesMatrix, 1, function(x) all(is.na(x[2:length(x)])))
#cytokinesMatrix = cytokinesMatrix[!ind,]

# cast table to numeric matrix
cytokinesNumericMatrix = data.matrix(cytokinesMatrix[,2:ncol(cytokinesMatrix)])

# scale data 
scaledCytokines = scale(cytokinesNumericMatrix)

# create data frame of scaled data
cytokines.df = data.frame(scaledCytokines)
cytokines.df$sample = factor(cytokinesMatrix$SampleID, levels=sampleorder)


cyto = ggplot(melt(cytokines.df, id.vars="sample"), aes(sample, variable, fill=value)) +
       geom_tile() +
       ylab("Cytokines") +
       scale_fill_gradient(low = "white", high = "steelblue") +
#       theme(axis.text.x = element_text(angle = 90, hjust = 1), legend.title=element_blank())
       theme(axis.text.x = element_blank(), legend.position="none", legend.title=element_blank(), axis.title.x = element_blank()) 

##############################################################
# 3. Generate heatmap for lipid data
#    Input Format:
#      SampleID LipidType Lipid Expression 
##############################################################

# reshape input into a table
lipidMatrix = dcast(lipids, SampleID ~ Lipid, value.var='Expression')

# cast data table into a matrix
lipidNumericMatrix = data.matrix(lipidMatrix[,2:ncol(lipidMatrix)])

# scale lipids 
scaledLipids = scale(lipidNumericMatrix)

# create data frame of scaled data
lipids.df = data.frame(scaledLipids)
lipids.df$sample = factor(lipidMatrix$SampleID, levels=sampleorder)

lpds = ggplot(melt(lipids.df, id.vars="sample"), aes(sample, variable, fill=value)) +
       geom_tile() +
       ylab("Lipids") +
       scale_fill_gradient(low = "white", high = "steelblue") +
       theme(axis.text.x = element_text(angle = 90, hjust = 1), legend.position="top", legend.title=element_blank(), axis.title.x = element_blank())
#       theme(axis.text.x = element_blank(), legend.position="none", legend.title=element_blank(), axis.title.x = element_blank()) 

# build image
gA <- ggplotGrob(cyto)
gB <- ggplotGrob(lpds)
gC <- ggplotGrob(sbars)

maxWidth = grid::unit.pmax(gA$widths[2:5], gB$widths[2:5], gC$widths[2:5])
gA$widths[2:5] <- as.list(maxWidth)
gB$widths[2:5] <- as.list(maxWidth)
gC$widths[2:5] <- as.list(maxWidth)
grid.arrange(gC, gA, gB, ncol=1)
